
#include <OpenSim/OpenSim.h>
#include "Simbody.h"

using namespace OpenSim;
using namespace SimTK;

class forceReport : public PeriodicEventReporter {

public:
	forceReport(const MultibodySystem& system,
		const CompliantContactSubsystem& contactForces,
		Real reportInterval)
		: PeriodicEventReporter(reportInterval), m_system(system), m_contactForces(contactForces) {}

	void handleEvent(const State& state) const override {

		m_system.realize(state, Stage::Dynamics);

		const int ncont = m_contactForces.getNumContactForces(state);

		std::cout << ncont << std::endl;

		for (int i = 0; i < ncont; ++i) {
			const ContactForce& force = m_contactForces.getContactForce(state, i);
			std::cout << force << std::endl;
		}
}
private:
	const MultibodySystem& m_system;
	const CompliantContactSubsystem& m_contactForces;
};


int main()
{
	try {
		
		Model osimModel("my_model.osim");		// Load the model from the OpenSim XML file "my_model.osim" and set its name to "osimModel"
		State& s = osimModel.initSystem();		// This builds the system from the OpenSim model and returns a writable model State.
		MultibodySystem& system = osimModel.updMultibodySystem();	// Returns a writable internal MultibodySystem created by the model.

		ContactTrackerSubsystem tracker(system);		// Creates a ContactTrackerSubsytem and adds it to the MultibodySystem "system".
		CompliantContactSubsystem contactForces(system, tracker);	// Creates a CompliantContactSubsytem and adds it to the MultibodySystem "system".

//		osimModel.buildSystem();					
		State& s_new = osimModel.initializeState();	// This finalizes the MultibodySystem (after the modifications made) and obtains an initial state from it.

		forceReport* conforceReporter = new forceReport(system, contactForces, 0.1);
		system.addEventReporter(conforceReporter);	// This includes the scheduled event reporter "conforceReporter" in the system.

		system.realizeTopology();				// 

		SimTK::RungeKuttaMersonIntegrator integrator(system);
		integrator.setAccuracy(1.0e-3);
		Manager manager(osimModel, integrator);

		manager.setInitialTime(0.0); manager.setFinalTime(1.0);
		manager.integrate(s_new);

		//CPodesIntegrator integ(system, CPodes::BDF, CPodes::Newton);
		//integ.setAccuracy(1e-3);
		//TimeStepper ts(system, integ);
		//ts.initialize(s_new);

	}
	catch (OpenSim::Exception ex)
	{
		std::cout << ex.getMessage() << std::endl;
		return 1;
	}
	catch (std::exception ex)
	{
		std::cout << ex.what() << std::endl;
		return 1;
	}
	catch (...)
	{
		std::cout << "UNRECOGNIZED EXCEPTION" << std::endl;
		return 1;
	}

	std::cout << "OpenSim example completed successfully.\n";
	std::cin.get();
	return 0;
}



